/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { contains, enterWith } from '../../support/cypress.util';
import { uploadFile } from '../../support/file-upload';
import { getAdjustedDateGerman } from '../../support/tech.util';

export class VorgangBescheidWizardE2EComponent {
  private readonly bewilligtButton: string = 'button-bewilligt';
  private readonly abgelehntButton: string = 'button-abgelehnt';
  private readonly ueberspringenButton: string = 'bescheid-ueberspringen';
  private readonly statusText: string = 'bescheid-status-text';
  private readonly ueberspringenDialog: string = 'bescheid-ueberspringen-dialog';
  private readonly ueberspringenAbbrechen: string = 'ueberspringen-abbrechen-button';
  private readonly ueberspringenAbschliessen: string = 'ueberspringen-abschliessen-button';
  private readonly dateInput: string = 'am-date-input';
  private readonly dateError: string = 'am-date-error';
  private readonly weiterButton: string = 'bescheid-weiter-button';
  private readonly stepCaption: string = 'step-cation';
  private readonly stepButton1: string = 'step-1-button';
  private readonly stepButton2: string = 'step-2-button';
  private readonly stepButton3: string = 'step-3-button';
  private readonly closeButton: string = 'close-bescheid';
  private readonly closeDialog: string = 'bescheid-close-dialog';
  private readonly bescheidResultBox: string = 'bescheiden-result';
  private readonly bescheidVerwerfenButton: string =
    'bescheiderstellung-abbrechen-entwurf-verwerfen';
  private readonly bescheidSpeichernButton: string =
    'bescheiderstellung-abbrechen-entwurf-speichern';
  private readonly uploadBescheidFileButton: string = '-single-file-upload-button';
  private readonly uploadAttachmentButton: string = 'Anhang_hochladen-file-upload-button';
  private readonly uploadAutomaticBescheid: string = 'create-bescheid-document-button';
  private readonly mailTextArea: string = 'Text-textarea-editor';
  private readonly saveBescheid: string = 'save-button';
  private readonly sendBescheid: string = 'send-button';
  private readonly confirmAndSaveButton: string = 'confirm-and-save-button';
  private readonly antragstellerText: string = 'bescheid-nachricht-an-antragsteller';
  private readonly betreffText: string = 'Betreff-text-input';
  private readonly nachrichtText: string = 'Text-textarea';

  private readonly fileBescheidValid: string = 'Bescheid_validpdf-file-item';
  private readonly fileAnhangValid: string = 'Anhang_validpdf-file-item';
  private readonly fileAutomaticBescheid: string =
    'KFAS_LIVE_KI_10_Haltverbot_befristetpdf-file-item';

  private readonly bescheidDocument: string = 'bescheid-document';
  private readonly attachmentDocument: string = 'bescheid-attachments';
  private readonly bescheidUploadSpinner: string =
    '[data-test-id="bescheid-document"] ods-spinner-icon';
  private readonly attachmentUploadSpinner: string =
    '[data-test-id="bescheid-attachments"] ods-spinner-icon';
  private readonly bescheidSaveSpinner: string =
    '[data-test-id="confirm-and-save-button"] ods-spinner-icon';
  private readonly sendenSpinner: string = '[data-test-id="send-button"] ods-spinner-icon';

  private locatorRoot: string = 'bescheid-wizard';

  private readonly bescheidVersenden: string = 'Bescheid versenden';
  private readonly dokumenteHinzufuegen: string = 'Dokumente hinzufügen';

  public getRoot(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.locatorRoot);
  }

  public getBewilligtButton(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.bewilligtButton);
  }

  public getAbgelehntButton(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.abgelehntButton);
  }

  public getUeberspringenButton(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.ueberspringenButton);
  }
  public getStatusText(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.statusText);
  }

  public getUeberspringenDialog(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.ueberspringenDialog);
  }

  public getUeberspringenAbbrechen(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.ueberspringenAbbrechen);
  }

  public getUeberspringenAbschliessen(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.ueberspringenAbschliessen);
  }

  public getDateInput(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.dateInput);
  }
  public getDateError(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.dateError);
  }

  public getWeiterButton(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.weiterButton);
  }

  public getStepCaption(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.stepCaption);
  }

  public getStepButton1(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.stepButton1);
  }

  public getStepButton2(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.stepButton2);
  }

  public getStepButton3(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.stepButton3);
  }

  public getCloseButton(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.closeButton);
  }

  public getCloseDialog(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.closeDialog);
  }

  public getCloseVerwerfenButton(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.bescheidVerwerfenButton);
  }

  public getCloseSpeichernButton(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.bescheidSpeichernButton);
  }

  public getUploadBescheidButton(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.uploadBescheidFileButton);
  }

  public getUploadAttachmentButton(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.uploadAttachmentButton);
  }

  public getFileBescheidValidInWizard(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.locatorRoot).find(`[data-test-id=${this.fileBescheidValid}]`);
  }

  public getFileAnhangValidInWizard(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.locatorRoot).find(`[data-test-id=${this.fileAnhangValid}]`);
  }

  public getAutomaticBescheidFileInWizard() {
    return cy.getTestElement(this.locatorRoot).find(`[data-test-id=${this.fileAutomaticBescheid}]`);
  }

  public getDeleteButtonOfElement(element: string): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(element).find('[title="Anhang löschen"]');
  }

  public getElementFromFileName(filename: string): string {
    return filename.replace(/\./g, '') + '-file-item';
  }

  public getBescheidDocument(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.bescheidDocument);
  }

  public getAttachmentDocument(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.attachmentDocument);
  }

  public getBescheidUploadSpinner(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.get(this.bescheidUploadSpinner);
  }

  public getAttachmentUploadSpinner(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.get(this.attachmentUploadSpinner);
  }

  public getBescheidSaveSpinner(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.get(this.bescheidSaveSpinner);
  }

  public getSendenSpinner(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.get(this.sendenSpinner);
  }

  public getMailText(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.nachrichtText);
  }

  public getSaveButton(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.saveBescheid);
  }

  public getSendButton(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.sendBescheid);
  }

  public getResultBox(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.bescheidResultBox);
  }

  public getConfirmAndSaveButton(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.confirmAndSaveButton);
  }

  public getAntragstellerText(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.antragstellerText);
  }

  public getBetreffText(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.betreffText);
  }

  public getNachrichtText(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.nachrichtText);
  }

  public isBescheidDocumentsStep(): void {
    contains(this.getRoot(), this.dokumenteHinzufuegen);
  }

  public isBescheidVersendenStep(): void {
    contains(this.getRoot(), this.bescheidVersenden);
  }

  public ablehnen(): void {
    this.getAbgelehntButton().should('exist').click();
  }

  public weiter(): void {
    this.getWeiterButton().should('exist').click();
  }

  public close(): void {
    this.getCloseButton().should('exist').click();
  }

  public isOpened(): void {
    this.getRoot().should('exist');
  }

  public isClosed(): void {
    this.getRoot().should('not.exist');
  }

  public getUploadAutomaticBescheidButton() {
    return cy.getTestElement(this.uploadAutomaticBescheid);
  }

  public uploadBescheid(fileName: string): void {
    uploadFile(this.getUploadBescheidButton(), fileName);
  }

  public uploadAttachment(fileName: string): void {
    uploadFile(this.getUploadAttachmentButton(), fileName);
  }

  public bescheidUploadSpinnerIsClosed(): void {
    this.getBescheidUploadSpinner().should('not.exist');
  }

  public attachmentSpinnerIsClosed(): void {
    this.getAttachmentUploadSpinner().should('not.exist');
  }

  public enterDate(difference: number): void {
    this.getDateInput().should('exist');
    enterWith(this.getDateInput(), getAdjustedDateGerman(difference));
  }

  public closeDialogIsShown(): void {
    this.getCloseDialog().should('exist');
  }
}
