/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  createEmptyStateResource,
  createStateResource,
  StateResource,
} from '@alfa-client/tech-shared';
import { createFilledSettingsListResource } from '../../test/admin-settings';
import { createPostfachResource } from '../../test/postfach/postfach';
import { SettingListResource } from './admin-settings.model';
import { getPostfachResource } from './admin-settings.util';
import { PostfachResource } from './postfach/postfach.model';

describe('get postfach resource', () => {
  it('should return state resource with postfach resource if exists', () => {
    const postfachResource: PostfachResource = createPostfachResource();
    const settingsListResource: StateResource<SettingListResource> = createStateResource(
      createFilledSettingsListResource([postfachResource]),
    );

    const postfachStateResource: StateResource<PostfachResource> =
      getPostfachResource(settingsListResource);

    expect(postfachStateResource.resource).toEqual(postfachResource);
  });

  it('should return empty state resource if postfach resource NOT exists', () => {
    const settingsListResource: StateResource<SettingListResource> = createStateResource(
      createFilledSettingsListResource([]),
    );

    const postfachStateResource: StateResource<PostfachResource> =
      getPostfachResource(settingsListResource);

    expect(postfachStateResource).toEqual(createEmptyStateResource());
  });
});
