/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BescheidService, BescheidWizardStep } from '@alfa-client/bescheid-shared';
import { Mock, existsAsHtmlElement, getElementFromFixtureByType, mock, triggerEvent, useFromMock } from '@alfa-client/test-utils';
import { EventEmitter } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MockComponent } from 'ng-mocks';
import { createBescheidResource } from '../../../../../../bescheid-shared/src/test/bescheid';
import { createCommandStateResource } from '../../../../../../command-shared/test/command';
import { getDataTestIdOf } from '../../../../../../tech-shared/test/data-test';
import { createVorgangWithEingangResource } from '../../../../../../vorgang-shared/test/vorgang';
import { BescheidWizardAbschliessenButtonComponent } from '../abschliessen-button/bescheid-wizard-abschliessen-button.component';
import { BescheidWizardStepTitleComponent } from '../step-title/bescheid-wizard-step-title.component';
import { BescheidWizardStepperComponent } from '../stepper/bescheid-wizard-stepper.component';
import { BescheidWizardSummaryComponent } from '../summary/bescheid-wizard-summary.component';
import { BescheidWizardAntragBescheidenComponent } from './bescheid-wizard-antrag-bescheiden.component';
import { BescheidWizardAntragBescheidenFormComponent } from './form/bescheid-wizard-antrag-bescheiden-form.component';
import { BescheidWizardAntragBescheidenSummaryComponent } from './summary/bescheid-wizard-antrag-bescheiden-summary.component';

describe('BescheidWizardAntragBescheidenComponent', () => {
  let component: BescheidWizardAntragBescheidenComponent;
  let fixture: ComponentFixture<BescheidWizardAntragBescheidenComponent>;

  const wizardStepper: string = getDataTestIdOf('wizard-stepper');
  const wizardStepTitle: string = getDataTestIdOf('wizard-step-title');
  const antragBescheidenForm: string = getDataTestIdOf('antrag-bescheiden-form');
  const abschliessenButton: string = getDataTestIdOf('wizard-abschliessen-button');
  const wizardSummary: string = getDataTestIdOf('wizard-summary');
  const antragBescheidenSummary: string = getDataTestIdOf('antrag-bescheiden-summary');

  let bescheidService: Mock<BescheidService>;

  beforeEach(() => {
    bescheidService = mock(BescheidService);
  });

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BescheidWizardAntragBescheidenComponent,
        MockComponent(BescheidWizardStepTitleComponent),
        MockComponent(BescheidWizardAntragBescheidenFormComponent),
        MockComponent(BescheidWizardSummaryComponent),
        MockComponent(BescheidWizardAntragBescheidenSummaryComponent),
        MockComponent(BescheidWizardAbschliessenButtonComponent),
        MockComponent(BescheidWizardStepperComponent),
      ],
      providers: [
        {
          provide: BescheidService,
          useValue: bescheidService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidWizardAntragBescheidenComponent);
    component = fixture.componentInstance;
    component.weiterClickEmitter = useFromMock(mock(EventEmitter<BescheidWizardStep>));
    component.vorgangAbgeschlossen = useFromMock(mock(EventEmitter<void>));
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('template', () => {
    describe('alfa-bescheid-wizard-stepper', () => {
      function getElementComponent(): BescheidWizardStepperComponent {
        return getElementFromFixtureByType(fixture, BescheidWizardStepperComponent);
      }

      it('should exists', () => {
        fixture.detectChanges();

        existsAsHtmlElement(fixture, wizardStepper);
      });

      describe('input', () => {
        it('should set activeStep', () => {
          fixture.detectChanges();

          expect(getElementComponent().activeStep).toBe(BescheidWizardStep.AntragBescheiden);
        });
      });
    });

    describe('alfa-bescheid-wizard-step-title', () => {
      it('should show', () => {
        existsAsHtmlElement(fixture, wizardStepTitle);
      });
    });

    describe('alfa-bescheid-wizard-summary', () => {
      it('should show', () => {
        existsAsHtmlElement(fixture, wizardSummary);
      });
    });

    describe('alfa-bescheid-wizard-antrag-bescheiden-summary', () => {
      it('should show', () => {
        existsAsHtmlElement(fixture, antragBescheidenSummary);
      });
    });

    describe('alfa-bescheid-wizard-abschliessen-button', () => {
      function getElementComponent(): BescheidWizardAbschliessenButtonComponent {
        return getElementFromFixtureByType(fixture, BescheidWizardAbschliessenButtonComponent);
      }

      it('should show', () => {
        existsAsHtmlElement(fixture, abschliessenButton);
      });

      describe('input', () => {
        it('should set vorgangWithEingangResource', () => {
          component.vorgangWithEingangResource = createVorgangWithEingangResource();

          fixture.detectChanges();

          expect(getElementComponent().vorgangWithEingangResource).toBe(component.vorgangWithEingangResource);
        });
      });

      describe('output', () => {
        it('should call onVorgangAbgeschlossen', () => {
          triggerEvent({
            fixture,
            name: 'vorgangAbgeschlossen',
            elementSelector: abschliessenButton,
          });

          expect(component.vorgangAbgeschlossen.emit).toHaveBeenCalled();
        });
      });
    });

    describe('alfa-bescheid-wizard-antrag-bescheiden-form', () => {
      function getElementComponent(): BescheidWizardAntragBescheidenFormComponent {
        return getElementFromFixtureByType(fixture, BescheidWizardAntragBescheidenFormComponent);
      }

      it('should show', () => {
        existsAsHtmlElement(fixture, antragBescheidenForm);
      });

      describe('input', () => {
        it('should set vorgangWithEingangResource', () => {
          component.vorgangWithEingangResource = createVorgangWithEingangResource();

          fixture.detectChanges();

          expect(getElementComponent().vorgangWithEingangResource).toBe(component.vorgangWithEingangResource);
        });

        it('should set bescheidResource', () => {
          component.bescheidResource = createBescheidResource();

          fixture.detectChanges();

          expect(getElementComponent().bescheidResource).toBe(component.bescheidResource);
        });

        it('should set submitStateResource', () => {
          component.submitStateResource = createCommandStateResource();

          fixture.detectChanges();

          expect(getElementComponent().submitStateResource).toBe(component.submitStateResource);
        });
      });

      describe('output', () => {
        it('should emit weiterClickEmitter', () => {
          triggerEvent({
            fixture,
            name: 'weiterClickEmitter',
            elementSelector: antragBescheidenForm,
          });

          expect(component.weiterClickEmitter.emit).toHaveBeenCalledWith(BescheidWizardStep.DokumenteHochladen);
        });
      });
    });
  });
});
