/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { faker } from '@faker-js/faker';
import { times } from 'lodash-es';
import { LinkRelationName } from '../../tech-shared/src';
import { toResource } from '../../tech-shared/test/resource';
import { OrganisationsEinheitResource } from '../../zustaendige-stelle-shared/src/lib/organisations-einheit/organisations-einheit.model';
import { CollaborationListLinkRel } from '../src';
import { Collaboration, CollaborationListResource, CollaborationResource } from '../src/lib/collaboration.model';

export function createCollaboration(): Collaboration {
  return {
    titel: faker.word.sample(2),
    beschreibung: faker.word.sample(10),
    zustaendigeStelle: faker.internet.url(),
  };
}

export function createCollaborationResource(linkRelations: LinkRelationName[] = []): CollaborationResource {
  return toResource(createCollaboration(), linkRelations);
}

export function createCollaborationResources(linkRelations: LinkRelationName[] = []): OrganisationsEinheitResource[] {
  return times(10, () => toResource(createCollaborationResource(), [...linkRelations]));
}

export function createCollaborationListResource(linkRelations: LinkRelationName[] = []): CollaborationListResource {
  return toResource({}, [...linkRelations], {
    [CollaborationListLinkRel.COLLABORATION_LIST]: createCollaborationResources(),
  });
}
