/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { argsToTemplate, moduleMetadata, type Meta, type StoryObj } from '@storybook/angular';

import { SaveIconComponent } from '../icons/save-icon/save-icon.component';
import { ButtonComponent } from './button.component';

const meta: Meta<ButtonComponent> = {
  title: 'Button',
  component: ButtonComponent,
  decorators: [
    moduleMetadata({
      imports: [ButtonComponent, SaveIconComponent],
    }),
  ],
  excludeStories: /.*Data$/,
  tags: ['autodocs'],
};

export default meta;
type Story = StoryObj<ButtonComponent>;

export const Default: Story = {
  args: {
    text: 'Hello world!',
    isLoading: false,
    variant: 'primary',
    size: 'medium',
    spinnerSize: 'medium',
  },
  argTypes: {
    variant: {
      options: ['primary', 'outline'],
      control: { type: 'radio' },
      table: { defaultValue: { summary: 'primary' } },
    },
  },
};

export const WithIcon: Story = {
  args: {
    text: 'I have an icon',
    variant: 'outline',
    size: 'medium',
  },
  render: (args) => ({
    props: args,
    template: `<ods-button ${argsToTemplate(args)}>
      <ods-save-icon icon size='small'></ods-save-icon>
    </ods-button>`,
  }),
};

export const IsLoading: Story = {
  args: {
    text: 'Loading...',
    isLoading: true,
    size: 'medium',
  },
};

export const OnlyIcon: Story = {
  args: {
    variant: 'icon',
    size: 'fit',
  },
  render: (args) => ({
    props: args,
    template: `<ods-button ${argsToTemplate(args)}>
      <ods-save-icon icon size='medium'></ods-save-icon>
    </ods-button>`,
  }),
};
