/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { mock, useFromMock } from '@alfa-client/test-utils';
import { VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { ResourceFactory } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { createForwardingListResource } from 'libs/forwarding-shared/test/forwarding';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { ForwardingListResource } from './forwarding.model';
import { ForwardingRepository } from './forwarding.repository';

describe('ForwardingRepository', () => {
  let repository: ForwardingRepository;
  let resourceFactory = mock(ResourceFactory);
  let resourceWrapper = { get: jest.fn() };

  beforeEach(() => {
    repository = new ForwardingRepository(useFromMock(resourceFactory));

    resourceFactory.fromId.mockReturnValue(resourceWrapper);
    resourceFactory.from.mockReturnValue(resourceWrapper);
  });

  describe('getForwardings', () => {
    const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource([
      VorgangWithEingangLinkRel.FORWARDING,
    ]);
    const commandList: ForwardingListResource = createForwardingListResource();

    beforeEach(() => {
      resourceWrapper.get.mockReturnValue(hot('a', { a: commandList }));
    });

    it('should call resourceFactory', () => {
      repository.getForwardings(vorgang);

      expect(resourceFactory.from).toHaveBeenCalledWith(vorgang);
    });

    it('should call resourceWrapper', () => {
      repository.getForwardings(vorgang);

      expect(resourceWrapper.get).toHaveBeenCalledWith(VorgangWithEingangLinkRel.FORWARDING);
    });

    it('should return result', () => {
      let result = repository.getForwardings(vorgang);

      expect(result).not.toBeNull();
      expect(result).toBeObservable(cold('a', { a: commandList }));
    });
  });
});
