/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { LOCALE_ID } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MAT_DATE_LOCALE } from '@angular/material/core';
import { MatIcon } from '@angular/material/icon';
import { ForwardingLinkRel } from '@alfa-client/forwarding-shared';
import { FormatDateWithTimePipe, HasLinkPipe } from '@alfa-client/tech-shared';
import { createForwardingResource } from 'libs/forwarding-shared/test/forwarding';
import { MockComponent } from 'ng-mocks';
import { VorgangForwardZustellenContainerComponent } from '../../../vorgang-forwarding-zustellen-container/vorgang-forward-zustellen-container.component';
import { VorgangForwardingInfoComponent } from './vorgang-forwarding-info.component';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('VorgangForwardingInfoComponent', () => {
  let component: VorgangForwardingInfoComponent;
  let fixture: ComponentFixture<VorgangForwardingInfoComponent>;

  const errorMessage: string = '[data-test-id="forwarding-error-message"]';
  const successMessage: string = '[data-test-id="forwarding-success-message"]';

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        VorgangForwardingInfoComponent,
        FormatDateWithTimePipe,
        MatIcon,
        HasLinkPipe,
        MockComponent(VorgangForwardZustellenContainerComponent),
      ],
      providers: [
        { provide: LOCALE_ID, useValue: 'de' },
        { provide: MAT_DATE_LOCALE, useValue: 'de-DE' },
      ],
    });
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(VorgangForwardingInfoComponent);
    component = fixture.componentInstance;
    component.forwarding = createForwardingResource();
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('on forwarding error', () => {
    it('should show error message', () => {
      component.forwarding = createForwardingResource([ForwardingLinkRel.ERROR]);
      fixture.detectChanges();

      const message = fixture.nativeElement.querySelector(errorMessage);

      expect(message).toBeInstanceOf(HTMLElement);
    });

    it('should hide successfull message', () => {
      component.forwarding = createForwardingResource([ForwardingLinkRel.ERROR]);
      fixture.detectChanges();

      const message = fixture.nativeElement.querySelector(successMessage);

      expect(message).not.toBeInstanceOf(HTMLElement);
    });
  });

  describe('on success forwarding', () => {
    it('should hide error message', () => {
      component.forwarding = createForwardingResource();
      fixture.detectChanges();

      const message = fixture.nativeElement.querySelector(errorMessage);

      expect(message).not.toBeInstanceOf(HTMLElement);
    });

    it('should show success message', () => {
      component.forwarding = createForwardingResource();
      fixture.detectChanges();

      const message = fixture.nativeElement.querySelector(successMessage);

      expect(message).toBeInstanceOf(HTMLElement);
    });
  });
});
