/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CommandOrder, CreateCommand } from '@alfa-client/command-shared';
import {
  createLoeschAnforderungZuruecknehmenCommand,
  createVorgangLoeschenCommand,
  createVorgangZumLoeschenMarkierenCommand,
  isLoeschAnforderungZuruecknehmen,
  isVorgangLoeschenCommand,
} from './loesch-anforderung.util';

describe('LoeschAnforderungUtil', () => {
  describe('create LoeschenAnfordernCommand', () => {
    it('should return command', () => {
      const command: CreateCommand = createVorgangZumLoeschenMarkierenCommand();

      expect(command.order).toBe(CommandOrder.VORGANG_ZUM_LOESCHEN_MARKIEREN);
    });
  });

  describe('create EndgueltigLoeschenCommand', () => {
    it('should return command', () => {
      const command: CreateCommand = createVorgangLoeschenCommand();

      expect(command.order).toBe(CommandOrder.VORGANG_LOESCHEN);
    });
  });

  describe('isVorgangLoeschenCommand', () => {
    it('should return true on VORGANG_LOESCHEN order', () => {
      const isCommand: boolean = isVorgangLoeschenCommand(CommandOrder.VORGANG_LOESCHEN);

      expect(isCommand).toBeTruthy();
    });

    it('should return true on other order', () => {
      const isCommand: boolean = isVorgangLoeschenCommand(CommandOrder.CREATE_KOMMENTAR);

      expect(isCommand).toBeFalsy();
    });
  });

  describe('createLoeschAnforderungZuruecknehmenCommand', () => {
    it('should return command', () => {
      const command: CreateCommand = createLoeschAnforderungZuruecknehmenCommand();

      expect(command.order).toBe(CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN);
    });
  });

  describe('isLoeschAnforderungZuruecknehmen', () => {
    it('should return true', () => {
      const result: boolean = isLoeschAnforderungZuruecknehmen(
        CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN,
      );

      expect(result).toBeTruthy();
    });

    it('should return false', () => {
      const result: boolean = isLoeschAnforderungZuruecknehmen(CommandOrder.EDIT_KOMMENTAR);

      expect(result).toBeFalsy();
    });
  });
});
