/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { FormBuilder, FormControl } from '@angular/forms';
import { mock, useFromMock } from '@alfa-client/test-utils';
import { VorgangService } from '@alfa-client/vorgang-shared';
import { EMPTY_STRING } from '@alfa-client/tech-shared';
import { createVorgangWithEingangResource } from '../../../../vorgang-shared/test/vorgang';
import { EMPTY } from 'rxjs';
import { AktenzeichenEditDialogFormservice } from './aktenzeichen-edit-dialog.formservice';

const vorgangService = mock(VorgangService);
const service = new AktenzeichenEditDialogFormservice(
  new FormBuilder(),
  useFromMock(vorgangService),
);

describe('getValue', () => {
  const formControl = new FormControl('aktenzeichen');
  jest.spyOn(service, 'getAktenzeichenFormControl').mockReturnValue(formControl);

  it('should return form control value', () => {
    const value = service.getValue();

    expect(value).toEqual('aktenzeichen');
  });

  it('should return trimmed value', () => {
    formControl.setValue('  aktenzeichen  ');

    const value = service.getValue();

    expect(value).toEqual('aktenzeichen');
  });

  it.each([null, EMPTY_STRING])('should return null', (aktenzeichen: string) => {
    formControl.setValue(aktenzeichen);

    const value = service.getValue();

    expect(value).toBeNull();
  });
});

describe('doSubmit', () => {
  it('should set aktenzeichen', () => {
    const vorgang = createVorgangWithEingangResource();
    service.setVorgangWithEingangResource(vorgang);
    jest.spyOn(service, 'getValue').mockReturnValue('aktenzeichen');
    vorgangService.setAktenzeichen.mockReturnValue(EMPTY);

    service.submit();

    expect(vorgangService.setAktenzeichen).toBeCalledWith(vorgang, 'aktenzeichen');
  });
});
