/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  BescheidLinkRel,
  BescheidResource,
  BescheidSendBy,
  BescheidService,
} from '@alfa-client/bescheid-shared';
import { StateResource, isLoaded, notHasLink } from '@alfa-client/tech-shared';
import { Component, OnInit } from '@angular/core';
import { Observable, filter, map, tap } from 'rxjs';
import { BescheidenFormService } from '../../../bescheiden.formservice';

@Component({
  selector: 'alfa-vorgang-detail-bescheiden-bescheid-versenden',
  templateUrl: './vorgang-detail-bescheiden-bescheid-versenden.component.html',
})
export class VorgangDetailBescheidenBescheidVersendenComponent implements OnInit {
  public bescheidResource$: Observable<BescheidResource>;

  public readonly formServiceClass = BescheidenFormService;
  public readonly sendBy = BescheidSendBy;

  public bescheidLinkRel = BescheidLinkRel;

  constructor(
    public formService: BescheidenFormService,
    private readonly bescheidService: BescheidService,
  ) {}

  ngOnInit(): void {
    this.bescheidResource$ = this.bescheidService.getBescheidDraft().pipe(
      filter(isLoaded),
      map((stateResource: StateResource<BescheidResource>) => stateResource.resource),
      tap((bescheidResource: BescheidResource) => this.initSendBy(bescheidResource)),
    );
  }

  initSendBy(bescheidResource: BescheidResource): void {
    if (notHasLink(bescheidResource, BescheidLinkRel.BESCHEIDEN_UND_SENDEN)) {
      this.formService.setSendBy(BescheidSendBy.MANUAL);
    }
  }
}
