/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BescheidService } from '@alfa-client/bescheid-shared';
import {
  createEmptyStateResource,
  createErrorStateResource,
  createStateResource,
} from '@alfa-client/tech-shared';
import { Mock, getElementFromFixture, mock } from '@alfa-client/test-utils';
import { VorgangService, VorgangWithEingangLinkRel } from '@alfa-client/vorgang-shared';
import { EventEmitter } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { createApiError } from 'libs/tech-shared/test/error';
import { MockComponent } from 'ng-mocks';
import { EMPTY, of } from 'rxjs';
import {
  createCommandResource,
  createCommandStateResource,
} from '../../../../../../../command-shared/test/command';
import { getDataTestIdOf } from '../../../../../../../tech-shared/test/data-test';
import { singleCold } from '../../../../../../../tech-shared/test/marbles';
import { createVorgangWithEingangResource } from '../../../../../../../vorgang-shared/test/vorgang';
import { BescheidenFormService } from '../../bescheiden.formservice';
import { VorgangDetailBescheidenStepTitleComponent } from '../vorgang-detail-bescheiden-step-title/vorgang-detail-bescheiden-step-title.component';
import { VorgangDetailBescheidenWeiterButtonComponent } from '../vorgang-detail-bescheiden-weiter-button/vorgang-detail-bescheiden-weiter-button.component';
import { VorgangDetailBescheidenAntragBescheidenComponent } from './vorgang-detail-bescheiden-antrag-bescheiden/vorgang-detail-bescheiden-antrag-bescheiden.component';
import { VorgangDetailBescheidenBescheidVersendenComponent } from './vorgang-detail-bescheiden-bescheid-versenden/vorgang-detail-bescheiden-bescheid-versenden.component';
import { VorgangDetailBescheidenDokumenteHinzufuegenComponent } from './vorgang-detail-bescheiden-dokumente-hinzufuegen/vorgang-detail-bescheiden-dokumente-hinzufuegen.component';
import { VorgangDetailBescheidenStepContentComponent } from './vorgang-detail-bescheiden-step-content/vorgang-detail-bescheiden-step-content.component';
import { VorgangDetailBescheidenStepsContentComponent } from './vorgang-detail-bescheiden-steps-content.component';
import { VorgangDetailBescheidenUeberspringenButtonComponent } from './vorgang-detail-bescheiden-ueberspringen-button/vorgang-detail-bescheiden-ueberspringen-button.component';

describe('VorgangDetailBescheidenStepsContentComponent', () => {
  let component: VorgangDetailBescheidenStepsContentComponent;
  let fixture: ComponentFixture<VorgangDetailBescheidenStepsContentComponent>;

  let vorgangService: Mock<VorgangService>;
  let bescheidService: Mock<BescheidService>;
  let formService: Mock<BescheidenFormService>;

  const bescheidVersendenTestId: string = getDataTestIdOf('bescheiden-bescheid-versenden');

  beforeEach(async () => {
    vorgangService = mock(VorgangService);
    vorgangService.getVorgangWithEingang.mockReturnValue(EMPTY);

    bescheidService = mock(BescheidService);
    formService = mock(BescheidenFormService);

    await TestBed.configureTestingModule({
      declarations: [
        VorgangDetailBescheidenStepsContentComponent,
        MockComponent(VorgangDetailBescheidenWeiterButtonComponent),
        MockComponent(VorgangDetailBescheidenStepTitleComponent),
        MockComponent(VorgangDetailBescheidenStepContentComponent),
        MockComponent(VorgangDetailBescheidenUeberspringenButtonComponent),
        MockComponent(VorgangDetailBescheidenAntragBescheidenComponent),
        MockComponent(VorgangDetailBescheidenDokumenteHinzufuegenComponent),
        MockComponent(VorgangDetailBescheidenBescheidVersendenComponent),
      ],
      providers: [
        {
          provide: VorgangService,
          useValue: vorgangService,
        },
        {
          provide: BescheidService,
          useValue: bescheidService,
        },
        {
          provide: BescheidenFormService,
          useValue: formService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(VorgangDetailBescheidenStepsContentComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('render', () => {
    it('should show documents component in step 2', () => {
      component.activeStep = 2;
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, getDataTestIdOf('bescheid-documents-upload'));

      expect(element).toBeInstanceOf(HTMLElement);
    });

    it.each([1, 3])('should not show documents component in step %d', (step) => {
      component.activeStep = step;
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, getDataTestIdOf('bescheid-documents-upload'));

      expect(element).not.toBeInstanceOf(HTMLElement);
    });

    it('should show bescheid versenden in step 3', () => {
      component.activeStep = 3;
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, bescheidVersendenTestId);

      expect(element).toBeInstanceOf(HTMLElement);
    });

    it.each([1, 2])('should not show bescheid versenden in step %d', (step: number) => {
      component.activeStep = step;
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, bescheidVersendenTestId);

      expect(element).not.toBeInstanceOf(HTMLElement);
    });
  });

  describe('changeActiveStep', () => {
    beforeEach(() => {
      component.activeStepChange = <any>mock(EventEmitter);
      formService.submit = jest.fn();
    });

    it('should submit', () => {
      component.canChangeTo = jest.fn().mockReturnValue(true);
      formService.submit = jest.fn().mockReturnValue(EMPTY);

      component.changeActiveStep(1);

      expect(formService.submit).toHaveBeenCalled();
    });

    it('should not submit', () => {
      component.canChangeTo = jest.fn().mockReturnValue(false);
      formService.submit = jest.fn().mockReturnValue(EMPTY);

      component.changeActiveStep(1);

      expect(formService.submit).not.toHaveBeenCalled();
    });
  });

  describe('canChangeTo', () => {
    it.each([1, 2])('should true for step %d', (step: number) => {
      const canChange = component.canChangeTo(step);

      expect(canChange).toBeTruthy();
    });

    it('should return true for step 3', () => {
      formService.validateBescheidDocumentExists.mockReturnValue(true);

      const canChange = component.canChangeTo(3);

      expect(canChange).toBeTruthy();
    });

    it('should return false for step 3', () => {
      formService.validateBescheidDocumentExists.mockReturnValue(false);

      const canChange = component.canChangeTo(3);

      expect(canChange).toBeFalsy();
    });
  });

  describe('ngOnInit', () => {
    it('should render überspringen und abschliessen button', () => {
      component.activeStep = 1;
      vorgangService.getVorgangWithEingang.mockReturnValue(
        of(
          createStateResource(
            createVorgangWithEingangResource([VorgangWithEingangLinkRel.ABSCHLIESSEN]),
          ),
        ),
      );

      component.ngOnInit();

      expect(component.showBescheidErstellungUeberspringen$).toBeObservable(singleCold(true));
    });

    it('should not render überspringen und abschliessen button for active step 2', () => {
      component.activeStep = 2;
      vorgangService.getVorgangWithEingang.mockReturnValue(
        of(
          createStateResource(
            createVorgangWithEingangResource([VorgangWithEingangLinkRel.ABSCHLIESSEN]),
          ),
        ),
      );

      component.ngOnInit();

      expect(component.showBescheidErstellungUeberspringen$).toBeObservable(singleCold(false));
    });

    it('should not render überspringen und abschliessen button if link missing', () => {
      component.activeStep = 1;
      vorgangService.getVorgangWithEingang.mockReturnValue(
        of(createStateResource(createVorgangWithEingangResource())),
      );

      component.ngOnInit();

      expect(component.showBescheidErstellungUeberspringen$).toBeObservable(singleCold(false));
    });
  });

  describe('noError', () => {
    it('should return true', () => {
      const noError = component.noError(createCommandStateResource());

      expect(noError).toBeTruthy();
    });

    it('should return false on not loaded state resource', () => {
      const noError = component.noError(createEmptyStateResource());

      expect(noError).toBeFalsy();
    });

    it('should return false on loading state resource', () => {
      const noError = component.noError(createStateResource(createCommandResource(), true));

      expect(noError).toBeFalsy();
    });

    it('should return false on error state resource', () => {
      const noError = component.noError(createErrorStateResource(createApiError()));

      expect(noError).toBeFalsy();
    });
  });
});
