/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BescheidLinkRel, BescheidService } from '@alfa-client/bescheid-shared';
import { StateResource, createEmptyStateResource, isLoaded } from '@alfa-client/tech-shared';
import { VorgangService, VorgangWithEingangLinkRel } from '@alfa-client/vorgang-shared';
import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { Resource, hasLink } from '@ngxp/rest';
import { Observable, filter, map, merge } from 'rxjs';
import { BescheidenFormService } from '../../bescheiden.formservice';

@Component({
  selector: 'alfa-vorgang-detail-bescheiden-weiter-button',
  templateUrl: './vorgang-detail-bescheiden-weiter-button.component.html',
})
export class VorgangDetailBescheidenWeiterButtonComponent implements OnInit {
  @Input() submitResource: StateResource<Resource> = createEmptyStateResource();
  @Output() clickEmitter: EventEmitter<MouseEvent> = new EventEmitter<MouseEvent>();

  showButton$: Observable<boolean>;

  constructor(
    public formService: BescheidenFormService,
    private readonly bescheidService: BescheidService,
    private readonly vorgangService: VorgangService,
  ) {}

  ngOnInit(): void {
    this.showButton$ = merge(
      this.vorgangService.getVorgangWithEingang(),
      this.bescheidService.getBescheidDraftIfExists(),
    ).pipe(
      filter(isLoaded),
      map(
        (stateResource) =>
          hasLink(stateResource.resource, VorgangWithEingangLinkRel.CREATE_BESCHEID_DRAFT) ||
          hasLink(stateResource.resource, BescheidLinkRel.UPDATE),
      ),
    );
  }
}
