/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.bescheid;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.Comparator;

import org.springframework.stereotype.Component;

@Component
class BescheidByBeschiedenAmComparator implements Comparator<Bescheid> {

	private static final DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");

	@Override
	public int compare(Bescheid bescheid1, Bescheid bescheid2) {
		var statusCompareResult = compareByStatus(bescheid1, bescheid2);
		if (statusCompareResult != 0) {
			return statusCompareResult;
		}

		var dateCompareResult = compareByBeschiedenAm(bescheid1, bescheid2);
		if (dateCompareResult != 0) {
			return dateCompareResult;
		}

		return compareBySentAt(bescheid1, bescheid2);
	}

	int compareByStatus(Bescheid bescheid1, Bescheid bescheid2) {
		if (BescheidStatus.DRAFT == bescheid1.getStatus()) {
			return 1;
		}
		if (BescheidStatus.DRAFT == bescheid2.getStatus()) {
			return -1;
		}
		return 0;
	}

	int compareByBeschiedenAm(Bescheid bescheid1, Bescheid bescheid2) {
		var date1 = LocalDate.parse(bescheid1.getBeschiedenAm(), dateTimeFormatter);
		var date2 = LocalDate.parse(bescheid2.getBeschiedenAm(), dateTimeFormatter);

		return date1.compareTo(date2);
	}

	int compareBySentAt(Bescheid bescheid1, Bescheid bescheid2) {
		return bescheid1.getSentInfo().getSentAt()
				.compareTo(bescheid2.getSentInfo().getSentAt());
	}

}
