/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.collaboration;

import java.util.Optional;

import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import lombok.RequiredArgsConstructor;

@RestController
@RequestMapping(FachstelleController.PATH)
@RequiredArgsConstructor
class FachstelleController {

	static final String PATH = "/api/fachstelles"; // NOSONAR
	static final String SEARCH_BY_PARAM = "searchBy";

	private final FachstelleService service;
	private final FachstelleModelAssembler modelAssembler;

	@GetMapping(params = { SEARCH_BY_PARAM })
	public CollectionModel<EntityModel<Fachstelle>> search(@RequestParam String searchBy) {
		return modelAssembler.toCollectionModel(service.searchFachstellen(searchBy).toList());
	}

	@GetMapping("/{id}")
	public ResponseEntity<EntityModel<Fachstelle>> getById(@PathVariable String id) {
		return ResponseEntity.of(Optional.of(service.getById(id)).map(modelAssembler::toModel));
	}
}
